#include <gtk/gtk.h>
#include <glade/glade.h>

#include <pthread.h>
#include <unistd.h>

#include "iwcommon.h"
#include "wavelan_support.h"
#include "scan_data.h"
#include "network_db.h"
#include "scan_dhcp.h"

/**************************************************
 some global definitions 
 **************************************************/

GladeXML *xml;
const char * wvlan_interface = "eth0";

typedef enum { NOT_SCANNING = 0, SCANNING } scan_state;

#define INTERFACE_FILE "skanner.glade"

/***************************************************************************
 prototypes
 ***************************************************************************/

void add_scan_data_to_clist(GtkCList *clist, scan_data_t *data);

/***************************************************************************
 the main skanner
 ***************************************************************************/
/* 
   set ESSID to nil to force skan for an ID
   wait a couple seconds then ask for the ESSID
   if it's set then lookup the MAC of our AP
   if it's new log it, otherwise log stats and reset/scan again

   TODO: add DHCP/NAT service detection
   TODO: maybe add GPS data
 */

static void *wavelan_skanzor(void * data)
{
  GtkWidget *clist = glade_xml_get_widget(xml, "clistMain");
  GtkWidget *statusbar = glade_xml_get_widget(xml, "stsMain");

  int sock = socket(AF_INET, SOCK_DGRAM, 0);
  
  while((*(int*)data) == SCANNING) {      
    /* main skanning loop - should lock this, but we'll just count on 
       one instance of it ever existing for now */

    /* check if essid is set - we might have found a network or already be 
       on one */

    if(wavelan_get_essid(sock, wvlan_interface)) {
      scan_data_t scan_data;

      gtk_statusbar_push(GTK_STATUSBAR(statusbar), 1, "Scanning... Have Signal");

      memset(&scan_data, 0, sizeof(scan_data));

      get_scan_data(sock, wvlan_interface, &scan_data);

      if(!network_db_exists(scan_data.ap_mac)) {
	gdk_beep();
      } 

//#error check this to see if it works
      if(scan_data.services.dhcp == 0) {
	/* lame */
//	system("/sbin/ifconfig eth0 down");
//	system("/sbin/ifconfig eth0 0.0.0.0 up");
//	system("/sbin/route add -host 255.255.255.255 eth0");
        
	/* start a search for DHCP services */
//	dhcp_search((void*)sock);
      }

      add_scan_data_to_clist(GTK_CLIST(clist), &scan_data);
      network_db_save_data(&scan_data);
      
      /* reset the ESSID */
      wavelan_set_essid(sock, wvlan_interface, NULL);
      
    } else {
      gtk_statusbar_push(GTK_STATUSBAR(statusbar), 1, "Scanning... No Signal");
    }

    /* sleep long enough to scan */
    sleep(3);
  }
  
  close(sock);
  pthread_exit(NULL);
}

void add_row_from_db(scan_data_t *row, void *user_data)
{
  GtkWidget * list = (GtkWidget*)user_data;
  add_scan_data_to_clist(GTK_CLIST(list), row);
}

void initialize_interface_from_db(void)
{
  GtkWidget *list = glade_xml_get_widget(xml, "clistMain");
  network_db_iterate(add_row_from_db, list);
}

static void
row_to_scan_data(GtkCList *clist, int row, scan_data_t *out)
{
  gchar *col_text = NULL;

  memset(out, 0, sizeof(scan_data_t));

  /* MAC */
  gtk_clist_get_text (clist, row, 0, &col_text);
  strcpy(out->essid, col_text);

  gtk_clist_get_text (clist, row, 1, &col_text);
  strcpy(out->ap_mac, col_text);

  gtk_clist_get_text (clist, row, 2, &col_text);
  out->link_quality = atoi(col_text);

  gtk_clist_get_text (clist, row, 4, &col_text);
  out->frequency = atof(col_text);

  gtk_clist_get_text (clist, row, 5, &col_text);
  strcpy(out->location, col_text);

}

/**************************************************
 interface support 
 **************************************************/

void handle_save_location(GtkWidget *w, gpointer user_data)
{
  GtkWidget *list_w = glade_xml_get_widget(xml, "clistMain");
  GtkCList  *clist   = GTK_CLIST(list_w);
  GtkWidget *entry_w = glade_xml_get_widget(xml, "txtLocation");
  GtkEntry *entry   = GTK_ENTRY(entry_w);

  gchar * location_text = gtk_entry_get_text(entry);

  if(clist->focus_row == -1) {
    printf("Can't save nothin'\n");
    return;
  }

  {
    scan_data_t data;

    row_to_scan_data(clist, clist->focus_row, &data);
    strcpy(data.location, location_text);
    network_db_save_data(&data);
    add_scan_data_to_clist(clist, &data);
  }

}

void handle_location_input(GtkWidget *widget, gpointer user_data) 
{
  const gchar * text = gtk_entry_get_text(GTK_ENTRY(widget));
  GtkWidget *statusbar = glade_xml_get_widget(xml, "stsMain");
  if(text && strlen(text) > 1024) {
    gtk_statusbar_push(GTK_STATUSBAR(statusbar), 1, "Text too long to save");
  } else {
    gtk_statusbar_push(GTK_STATUSBAR(statusbar), 1, "");
  }
}

void handle_clist_selection(GtkWidget *w, gpointer user_data)
{
  GtkWidget *ltext = glade_xml_get_widget(xml, "txtLocation");
  GtkCList *list = GTK_CLIST(w);
  gchar * location_text = NULL;
  
  gtk_clist_get_text(list, list->focus_row, 5, &location_text);

  gtk_entry_set_text(GTK_ENTRY(ltext), location_text);
}

static int
find_record_or_new_row(GtkCList *clist, const char *mac)
{
  int i = 0;

  if(clist->rows == 0)
    return -1;

  /* FIXME: this will not scale to anything reasonable */
  for(i = 0; i < clist->rows; i++) {
    gchar *text;
    
    gtk_clist_get_text (clist, i, 1, &text);

    if(strcmp(text, mac) == 0) {
      return i;
    }
  }

  return -1;
}

void
add_scan_data_to_clist(GtkCList *clist, scan_data_t *data)
{
  gchar *row[] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL };
  int row_num = find_record_or_new_row(clist, data->ap_mac);

  char signal_buf[3], avg_signal_buf[10], freq[32];

  sprintf(signal_buf, "%d", data->link_quality);
  if(data->link_samples != 0) {
    sprintf(avg_signal_buf, "%d", (int)(data->link_total / data->link_samples));
  } else {
    memset(avg_signal_buf, 0, sizeof(avg_signal_buf));
  } 

  sprintf(freq, "%2.5f", data->frequency / (1024 * 1024 * 1024));

  row[0] = data->essid;
  row[1] = data->ap_mac;
  row[2] = signal_buf;
  row[3] = avg_signal_buf;
  row[4] = freq;
  row[5] = (data->location[0] == '\0' ? "Not Known" : data->location);
  
  if(row_num == -1) {
    gtk_clist_insert(clist, row_num, row);
  } else {
    gtk_clist_set_text(clist, row_num, 2, row[2]);
    gtk_clist_set_text(clist, row_num, 3, row[3]);
    gtk_clist_set_text(clist, row_num, 5, row[5]);
  }

}

/* how bogus is this? - it could be a lot worse, trust me */
void set_button_text(GtkButton * button, const gchar * text)
{
  GList * child_list = gtk_container_children(GTK_CONTAINER(button));

  gtk_label_set(
		gtk_container_children(GTK_CONTAINER(child_list->data))->data, text);
}

void start_or_stop_scan(GtkWidget *widget, gpointer user_data) 
{
  GtkWidget *statusbar = NULL;

  static scan_state state = NOT_SCANNING;
  const gchar * label_text = NULL;
  const gchar * status_text = NULL;

  GtkButton * scan_button = GTK_BUTTON(user_data);
  
  switch(state)
    {
    case SCANNING:
      state = NOT_SCANNING;
      label_text = "Scan";
      status_text = "";
      break;
    case NOT_SCANNING:
      {
	pthread_t skannor_thread;
	
	state = SCANNING;
	label_text = "Stop";
	status_text = "Scanning...";
	pthread_create(&skannor_thread, NULL, wavelan_skanzor, &state);
      }
      break;
    }
  
  set_button_text(scan_button, label_text);
  statusbar = glade_xml_get_widget(xml, "stsMain");
  gtk_statusbar_push(GTK_STATUSBAR(statusbar), 1, status_text);
}

void main_window_exit(GtkWidget *widget, gpointer user_data) 
{
  gtk_main_quit();
}

int main(int argc, char *argv[]) 
{
  GtkWidget * foo = NULL;
  
  gtk_init(&argc, &argv);
  glade_init();
  
  /* load the interface */
  xml = glade_xml_new(INTERFACE_FILE, NULL);
  
  /* connect the signals in the interface */
  glade_xml_signal_autoconnect(xml);
  
  foo = glade_xml_get_widget(xml, "wndMain");
  
  gtk_signal_connect (GTK_OBJECT(foo), "delete-event", 
		      (GtkSignalFunc) main_window_exit, NULL);

  if(getuid() != 0) {
    g_print("Scanning not available, you must be root to scan\n");

    foo = glade_xml_get_widget(xml, "btnStartStop");
    gtk_widget_set_sensitive(foo, FALSE);

    foo = glade_xml_get_widget(xml, "stsMain");
    gtk_statusbar_push(GTK_STATUSBAR(foo), 1, "You must be root to scan");
  }

  initialize_interface_from_db();

  /* start the event loop */
  gtk_main();
  return 0;
}
